/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef PCAP_FRAME_ITERATOR_HH_
# define PCAP_FRAME_ITERATOR_HH_

# include <istream>

# include <wipal/pcap/frame_descriptor.hh>
# include <wipal/pcap/descriptor.hh>
# include <wipal/tool/valued_iterator.hh>


/*---------------------.
| Forward declarations |
`---------------------*/

namespace pcapxx
{

  template <class>
  struct descriptor;

  /// WiPal PCAP internals - not designed for users.
  namespace internals
  {

    template <class>
    struct frame_iterator;

  } // End of namespace pcapxx::internals.

} // End of namespace pcapxx.


/*--------------.
| Virtual types |
`--------------*/

namespace tool
{

  /// Virtual %types for pcapxx::internals::frame_iterator.  @see types
  template <class B>
  struct types< pcapxx::internals::frame_iterator<B> >
  {
    typedef pcapxx::frame_descriptor	value_type;
    typedef pcapxx::descriptor<bottom>	iterable_type;
  };

} // End of namespace tool.

/*-------------.
| Declarations |
`-------------*/

namespace pcapxx
{
  namespace internals
  {

    /// Iterator over frames for @link descriptor PCAP descriptors @endlink.
    template <class Bottom = tool::bottom>
    struct frame_iterator:
      WP_INHERIT(public tool::valued_iterator, frame_iterator<Bottom>)
    {
      /// @name Static hierarchies related types
      //@{
      typedef WP_GET_EXACT(Bottom, frame_iterator<Bottom>)	exact_type;
      typedef tool::valued_iterator<exact_type>			super_type;
      typedef WP_TYPE(iterable_type, exact_type)		iterable_type;
      typedef WP_TYPE(value_type, exact_type)			value_type;
      //@}

      /// tool::iterable compatible constructor.
      frame_iterator(const iterable_type& desc, bool end);

      /// Construct a frame iterator at a given PCAP file position.
      frame_iterator(const iterable_type&	desc,
		     const std::streampos&	pos,
		     unsigned			count);

      /// Copy constructor.
      frame_iterator(const frame_iterator&);

      /// Assignment operator.
      frame_iterator&	operator = (const frame_iterator&);

      /// @name Abstract method implementations
      //@{
      bool		equal(const frame_iterator& rhs) const;
      void		increment();
      //@}

    private:

      /// Retrieve the underlying C++ input stream.
      std::istream&	stream();

      /*-----------.
      | Attributes |
      `-----------*/

      /// Link to the descriptor this iterator comes from.
      const iterable_type*		desc_;

      /**
       * @name Stream-related information
       *
       * If @c this->value() holds @c boost::none_t (i.e. the iterator
       * is at the end), do not expect those fields to have
       * a meaningful value!
       */
      //@{

      /*
       * Profiling showed std::ifstream's constructor takes much time,
       * even when not requesting to open a file (which is a frequent
       * case).  Thus we use a pointer instead to avoid that overhead.
       * Of course, this implies the required allocation will slow
       * down cases where we must open a file and initialize this
       * attribute.
       */
      std::auto_ptr<std::istream>	input_;

      std::streampos			pos_;	///<< Current file offset.
      unsigned				count_;	///<< Current packet id.

      //@}
    };

  } // End of namespace pcapxx::internals.

} // End of namespace pcapxx.

# include "frame_iterator.hxx"

#endif // ! PCAP_FRAME_ITERATOR_HH_
