/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef PRISM_HEADER_HH_
# define PRISM_HEADER_HH_

# include <wipal/phy/phy.hh>

// Forward declarations.
namespace prism
{
  struct	time;
  struct	header;
}


// Virtual types.
namespace tool
{

  template <>
  struct types<prism::time>
  {
    typedef uint32_t	impl_type;
  };

  template <>
  struct types<prism::header>
  {
    typedef prism::time	time_type;
  };

} // End of namespace tool.


// Core declarations.

/// Prism %header related compounds.
namespace prism
{
  /**
   * @link header Prism %header @endlink %item.
   *
   * @see prism::header
   */
  struct item
  {
    /// Read the item's value according to given endianness.
    uint32_t	get(bool swapped) const;

    /// Set the item's value according to given endianness.
    item&	set(uint32_t value, bool swapped);

  protected:
    uint32_t did;	///< What this item's value is.
    uint16_t status;	///< Is this item's value present (0) or absent (1).
    uint16_t len;	///< Length in bytes, should be 4.
    uint32_t data;	///< Item value or zero.
  };

  /// Prism headers timing information.  @see phy::time
  struct time: public phy::time<prism::time>
  {
    /// @name Static hierarchies related types and methods.
    //@{
    typedef prism::time				exact_type;
    typedef WP_TYPE_(impl_type, exact_type)	impl_type;

    impl_type	get_impl() const;
    exact_type&	increment(tool::microseconds microseconds);
    //@}

    /// Default constructor.  Initialize all fields to zero.
    time();

    /// Initialize mactime and hosttime according to the given arguments.
    time(uint32_t mactime, uint32_t hosttime);

    /// Retrieve the header's MAC time.
    uint32_t	mactime() const;

    /// Retrieve the header's host time.
    uint32_t	hosttime() const;

  private:
    uint32_t	mactime_;
    uint32_t	hosttime_;
  };

  /// Prism %header format.
  struct header: public phy::header<prism::header>
  {
    /// @name Static hierarchies related types and methods.
    //@{
    typedef prism::header			exact_type;
    typedef WP_TYPE_(time_type, exact_type)	time_type;

    size_t	len_impl(size_t, bool) const;
    time_type	time_get_impl(bool) const;
    void	time_set_impl(const time_type&, bool);
    static bool	eq_time(const pcapxx::frame_descriptor& lhs,
			const pcapxx::frame_descriptor& rhs,
			tool::endian::endianness	phy_end,
			unsigned			prec);
    //@}

    enum
      {
	/// @link header::devname @c devname @endlink's maximum length.
	devnamelen_max = 16
      };

    uint32_t	msgcode;			///< Message code.
    uint32_t	msglen;				///< Header length.
    uint8_t	devname[devnamelen_max];	///< Interface name.
    item	hosttime;			///< Host time.
    item	mactime;			///< MAC time.
    item	channel;			///< Channel.
    item	rssi;				///< RSSI.
    item	sq;				///< Signal quality(?).
    item	signal;				///< Signal quality(?).
    item	noise;				///< Noise information.
    item	rate;				///< Tx/Rx rate.
    item	istx;				///< Is TX.
    item	frmlen;				///< Frame length.
  };

} // End of namespace prism.

# include "prism_header.hxx"

#endif // ! PRISM_HEADER_HH_
