/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */

#include <iostream>
#include <stdexcept>
#include <cstdlib>

#include <boost/format.hpp>
#include <boost/foreach.hpp>
#include <boost/preprocessor/tuple/elem.hpp>
#include <boost/preprocessor/seq/for_each.hpp>

#include "config.hh"

#include <wipal/tool/options.hh>
#include <wipal/tool/microseconds_stamp.hh>
#include <wipal/wifi/frame/filter/uniquely_identifiable.hh>
#include <wipal/wifi/addr_mapping.hh>
#include <wipal/pcap/list_of_traces.hh>

namespace
{

  template <class UniqueId>
  struct unique_frame_dumper
  {
    unique_frame_dumper(const tool::opt::options& options):
      options_ (options),
      format_ ("%20s")
    {
    }

    void	operator () (const tool::microseconds_stamp<UniqueId>& frame)
    {
      std::cout << (format_ % frame);
      if (options_.print_microseconds)
	std::cout << '\t' << (format_ % frame.microseconds());
      std::cout << '\n';
    }

    void	print_colheads()
    {
      std::cout << (format_ % "Frame");
      if (options_.print_microseconds)
	std::cout << '\t' << (format_ % "Time (us)");
      std::cout << '\n'

		<< (format_ % "=====");
      if (options_.print_microseconds)
	std::cout << '\t' << (format_ % "=========");
      std::cout << std::endl;
    }

  private:
    const tool::opt::options&	options_;
    boost::format		format_;
  };

  struct main_helper
  {
    template <class UniqueId>
    int
    operator () (const tool::opt::options& options) const
    {
      using wifi::frame::filter::for_each_uniquely_identifiable;

      wifi::addr_mapping		mapping (options.mapping);
      unique_frame_dumper<UniqueId>	dump (options);

      BOOST_FOREACH(pcapxx::list_of_traces<> traces, options.input)
	{
	  if (options.prism_noise_means_error)
	    traces.expect(pcapxx::descriptor<>::PRISM_HEADER);

	  if (options.print_column_headers)
	    {
	      std::cout << traces.file_name() << ":\n";
	      dump.print_colheads();
	    }

	  if (not traces.empty())
	    switch (traces.iterable(0).linktype())
	      {

#define WP_case(R, Unused, LT)						\
	      case pcapxx::descriptor<>::BOOST_PP_TUPLE_ELEM(2, 0, LT):	\
		traces.expect(pcapxx::descriptor<>::			\
			      BOOST_PP_TUPLE_ELEM(2, 0, LT));		\
	        for_each_uniquely_identifiable<UniqueId,		\
		  BOOST_PP_TUPLE_ELEM(2, 1, LT)>			\
		  (traces.begin(), traces.end(), mapping, dump,		\
		   options.prism_noise_means_error,			\
		   options.endianness);					\
		break;

		BOOST_PP_SEQ_FOR_EACH(WP_case, ~, WP_LINKTYPES)
#undef WP_case

	      default:
		  tool::opt::
		  unsupported_linktype(traces.iterable(0).linktype());
	      }
	}
      return EXIT_SUCCESS;
    }

  }; // End of struct main_helper.

} // End of anonymous namespace.

int
main(int argc, char *const argv[])
{
  try
    {
      tool::opt::options options =
	tool::opt::parse_args(argc, argv,
			      WP_OPT_ATTR
			      WP_OPT_COLHEADS
			      WP_OPT_MAPPING
			      WP_OPT_ENDIANNESS
			      WP_OPT_PRISM_NOISE
			      WP_OPT_USEC,
			      tool::opt::options::arbitrary_number_of_inputs,
			      tool::opt::options::no_output);
      return tool::opt::dispatch_unique_id(options, main_helper ());
    }
  catch (const std::exception& e)
    {
      std::cerr << "ERROR: " << e.what() << '.' << std::endl;
      return EXIT_FAILURE;
    }
}
