/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#include <set>
#include <vector>
#include <map>
#include <utility>
#include <iostream>
#include <stdexcept>
#include <cstdlib>

#include <boost/format.hpp>
#include <boost/foreach.hpp>
#include <boost/preprocessor/tuple/elem.hpp>
#include <boost/preprocessor/seq/for_each.hpp>


#include "config.hh"

#include <wipal/tool/microseconds.hh>
#include <wipal/tool/microseconds_stamp.hh>
#include <wipal/tool/options.hh>
#include <wipal/wifi/addr_mapping.hh>
#include <wipal/pcap/list_of_traces.hh>
#include <wipal/wifi/frame/filter/uniquely_identifiable.hh>

namespace
{

  template <class UniqueID>
  class uniqueness_tester
  {

    struct collision
    {
      collision(const unsigned rhs_id, const tool::microseconds& interval):
	rhs_id (rhs_id),
	interval (interval)
      {
      }

      unsigned			rhs_id;
      tool::microseconds	interval;
    };

    typedef tool::microseconds_stamp<UniqueID>		arrival;
    typedef std::set<arrival>				set_of_arrivals;
    typedef std::vector<collision>			collisions_vector;
    typedef std::map<arrival, collisions_vector>	collisions_map;

    const tool::opt::options&	options_;
    set_of_arrivals		arrivals_;
    collisions_map		collisions_;
    unsigned			arrival_count_;
    unsigned			collision_count_;

  public:

    uniqueness_tester(const tool::opt::options& options):
      options_ (options),
      arrival_count_ (0),
      collision_count_ (0)
    {
    }

    void		operator () (const arrival& a)
    {
      typedef typename set_of_arrivals::iterator iterator;

      const std::pair<iterator, bool> r = arrivals_.insert(a);

      ++arrival_count_;
      if (not r.second)
	{
	  const tool::microseconds dt (a.microseconds() -
				       r.first->microseconds());

	  ++collision_count_;
	  collisions_[*r.first].push_back(collision (a.frame_id(), dt));
	}
    }

    void	print(std::ostream& columns, std::ostream& summary) const
    {
      typedef typename collisions_vector::const_iterator	cvec_iterator;
      typedef typename collisions_map::const_iterator		cmap_iterator;

      boost::format f ("%15s %15s %15s");

      if (options_.print_column_headers)
	{
	  columns << (f % "frame 1" % "frame 2" % "interval (us)") << '\n';
	  columns << (f % "=======" % "=======" % "=============") << std::endl;
	}
      for (cmap_iterator i = collisions_.begin(); i != collisions_.end(); ++i)
	{
	  bool first = true;
	  for (cvec_iterator j = i->second.begin(); j != i->second.end(); ++j)
	    {
	      columns << (f % i->first % j->rhs_id % j->interval);
	      if (first)
		{
		  first = false;
		  columns << " (new)";
		}
	      columns << '\n';
	    }
	}
      if (options_.print_summary)
	summary << arrival_count_ << " unique frames, "
		<< collision_count_ << " collisions." << std::endl;
    }

    unsigned	collision_count() const
    {
      return collision_count_;
    }

  };


  struct main_helper
  {
    template <class UniqueId>
    int
    operator () (const tool::opt::options& options) const
    {
      wifi::addr_mapping	mapping (options.mapping);
      bool			success (true);

      BOOST_FOREACH(pcapxx::list_of_traces<> traces, options.input)
	{
	  using wifi::frame::filter::for_each_uniquely_identifiable;

	  uniqueness_tester<UniqueId>	tester (options);

	  if (options.prism_noise_means_error)
	    traces.expect(pcapxx::descriptor<>::PRISM_HEADER);

	  if (not traces.empty())
	    switch (traces.iterable(0).linktype())
	      {

#define WP_case(R, Unused, LT)						\
	      case pcapxx::descriptor<>::BOOST_PP_TUPLE_ELEM(2, 0, LT):	\
		traces.expect(pcapxx::descriptor<>::			\
			      BOOST_PP_TUPLE_ELEM(2, 0, LT));		\
	        for_each_uniquely_identifiable<UniqueId,		\
		  BOOST_PP_TUPLE_ELEM(2, 1, LT)>			\
		  (traces.begin(), traces.end(), mapping, tester,	\
		   options.prism_noise_means_error,			\
		   options.endianness);					\
		break;

		BOOST_PP_SEQ_FOR_EACH(WP_case, ~, WP_LINKTYPES)
#undef WP_case

	      default:
		tool::opt::unsupported_linktype(traces.iterable(0).linktype());
	      }

	  if (options.print_column_headers)
	    std::cout << traces.file_name() << ":\n";
	  tester.print(std::cout, std::cerr);
	  if (tester.collision_count())
	    success = false;
	}

      return success ? EXIT_SUCCESS : EXIT_FAILURE;
    }
  }; // End of struct main_helper.

} // End of anonymous namespace.

int
main(int argc, char *const argv[])
{
  try
    {
      const tool::opt::options options =
	tool::opt::parse_args(argc, argv,
			      WP_OPT_ATTR
			      WP_OPT_COLHEADS
			      WP_OPT_MAPPING
			      WP_OPT_ENDIANNESS
			      WP_OPT_PRISM_NOISE
			      WP_OPT_QUIET,
			      tool::opt::options::arbitrary_number_of_inputs,
			      tool::opt::options::no_output);
      return tool::opt::dispatch_unique_id(options, main_helper ());
    }
  catch (const std::exception& e)
    {
      std::cerr << "ERROR: " << e.what() << '.' << std::endl;
      return EXIT_FAILURE;
    }
}
